import { CommandMetadata, CommandMetadataInput, CommandMetadataOption, HydratedCommandMetadata, HydratedNamespaceMetadata, ICommand, INamespace, NamespaceLocateResult, NamespaceMetadata } from '../definitions';
import { Colors } from './colors';
export declare enum CommandGroup {
    Deprecated = "deprecated",
    Hidden = "hidden",
    Beta = "beta",
    Experimental = "experimental"
}
export declare enum NamespaceGroup {
    Deprecated = "deprecated",
    Hidden = "hidden",
    Beta = "beta",
    Experimental = "experimental"
}
export declare enum OptionGroup {
    Deprecated = "deprecated",
    Hidden = "hidden",
    Beta = "beta",
    Experimental = "experimental",
    Advanced = "advanced"
}
export declare function isOptionVisible<O extends CommandMetadataOption>(opt: O): Promise<boolean>;
export declare function isCommandVisible<C extends ICommand<C, N, M, I, O>, N extends INamespace<C, N, M, I, O>, M extends CommandMetadata<I, O>, I extends CommandMetadataInput, O extends CommandMetadataOption>(cmd: HydratedCommandMetadata<C, N, M, I, O>): Promise<boolean>;
export declare abstract class HelpFormatter {
    protected readonly colors: Colors;
    constructor({ colors }: {
        colors?: Colors;
    });
    abstract format(): Promise<string>;
}
export interface NamespaceHelpFormatterDeps<C extends ICommand<C, N, M, I, O>, N extends INamespace<C, N, M, I, O>, M extends CommandMetadata<I, O>, I extends CommandMetadataInput, O extends CommandMetadataOption> {
    readonly location: NamespaceLocateResult<C, N, M, I, O>;
    readonly namespace: N;
    readonly colors?: Colors;
}
export declare abstract class NamespaceHelpFormatter<C extends ICommand<C, N, M, I, O>, N extends INamespace<C, N, M, I, O>, M extends CommandMetadata<I, O>, I extends CommandMetadataInput, O extends CommandMetadataOption> extends HelpFormatter {
    protected readonly location: NamespaceLocateResult<C, N, M, I, O>;
    protected readonly namespace: N;
    protected readonly dotswidth: number;
    protected _metadata?: NamespaceMetadata;
    protected _fullName?: string;
    constructor({ location, namespace, colors }: NamespaceHelpFormatterDeps<C, N, M, I, O>);
    /**
     * Given command metadata, decide whether to keep or discard the command that
     * the metadata represents.
     *
     * @param meta: The metadata of the command.
     * @return `true` to keep, `false` to discard
     */
    filterCommandCallback(meta: HydratedCommandMetadata<C, N, M, I, O>): Promise<boolean>;
    getNamespaceMetadata(): Promise<NamespaceMetadata>;
    getNamespaceFullName(): Promise<string>;
}
export declare class NamespaceStringHelpFormatter<C extends ICommand<C, N, M, I, O>, N extends INamespace<C, N, M, I, O>, M extends CommandMetadata<I, O>, I extends CommandMetadataInput, O extends CommandMetadataOption> extends NamespaceHelpFormatter<C, N, M, I, O> {
    formatHeader(): Promise<string>;
    formatSummary(): Promise<string>;
    formatDescription(): Promise<string>;
    getGlobalOptions(): Promise<string[]>;
    formatUsage(): Promise<string>;
    formatCommands(): Promise<string>;
    formatCommandGroup(title: string, commands: ReadonlyArray<HydratedCommandMetadata<C, N, M, I, O>>): Promise<string>;
    getListOfCommandDetails(commands: ReadonlyArray<HydratedCommandMetadata<C, N, M, I, O>>): Promise<string[]>;
    getListOfNamespaceDetails(commands: ReadonlyArray<HydratedCommandMetadata<C, N, M, I, O>>): Promise<string[]>;
    /**
     * Insert text before the namespace's summary.
     *
     * @param meta: The metadata of the namespace.
     */
    formatBeforeSummary(meta: NamespaceMetadata): Promise<string>;
    /**
     * Insert text after the namespace's summary.
     *
     * @param meta: The metadata of the namespace.
     */
    formatAfterSummary(meta: NamespaceMetadata): Promise<string>;
    /**
     * Insert text that appears before a commands's summary.
     *
     * @param meta: The metadata of the command.
     */
    formatBeforeCommandSummary(meta: HydratedCommandMetadata<C, N, M, I, O>): Promise<string>;
    /**
     * Insert text that appears after a commands's summary.
     *
     * @param meta: The metadata of the command.
     */
    formatAfterCommandSummary(meta: HydratedCommandMetadata<C, N, M, I, O>): Promise<string>;
    /**
     * Insert text that appears before a namespace's summary.
     *
     * @param meta The metadata of the namespace.
     * @param commands An array of the metadata of the namespace's commands.
     */
    formatBeforeNamespaceSummary(meta: HydratedNamespaceMetadata<C, N, M, I, O>, commands: ReadonlyArray<HydratedCommandMetadata<C, N, M, I, O>>): Promise<string>;
    /**
     * Insert text that appears after a namespace's summary.
     *
     * @param meta The metadata of the namespace.
     * @param commands An array of the metadata of the namespace's commands.
     */
    formatAfterNamespaceSummary(meta: HydratedNamespaceMetadata<C, N, M, I, O>, commands: ReadonlyArray<HydratedCommandMetadata<C, N, M, I, O>>): Promise<string>;
    format(): Promise<string>;
}
export interface CommandHelpFormatterDeps<C extends ICommand<C, N, M, I, O>, N extends INamespace<C, N, M, I, O>, M extends CommandMetadata<I, O>, I extends CommandMetadataInput, O extends CommandMetadataOption> {
    readonly location: NamespaceLocateResult<C, N, M, I, O>;
    readonly command: C;
    /**
     * Provide extra context with hydrated command metadata. If not provided,
     * `command.getMetadata()` is called.
     */
    readonly metadata?: HydratedCommandMetadata<C, N, M, I, O>;
    readonly colors?: Colors;
}
export declare abstract class CommandHelpFormatter<C extends ICommand<C, N, M, I, O>, N extends INamespace<C, N, M, I, O>, M extends CommandMetadata<I, O>, I extends CommandMetadataInput, O extends CommandMetadataOption> extends HelpFormatter {
    protected readonly location: NamespaceLocateResult<C, N, M, I, O>;
    protected readonly command: C;
    protected readonly dotswidth: number;
    protected _metadata?: M;
    protected _hydratedMetadata?: HydratedCommandMetadata<C, N, M, I, O>;
    protected _fullName?: string;
    constructor({ location, command, metadata, colors }: CommandHelpFormatterDeps<C, N, M, I, O>);
    /**
     * Given an option definition from command metadata, decide whether to keep
     * or discard it.
     *
     * @return `true` to keep, `false` to discard
     */
    filterOptionCallback(option: O): Promise<boolean>;
    getCommandMetadata(): Promise<M | HydratedCommandMetadata<C, N, M, I, O>>;
    getCommandFullName(): Promise<string>;
}
export declare class CommandStringHelpFormatter<C extends ICommand<C, N, M, I, O>, N extends INamespace<C, N, M, I, O>, M extends CommandMetadata<I, O>, I extends CommandMetadataInput, O extends CommandMetadataOption> extends CommandHelpFormatter<C, N, M, I, O> {
    formatHeader(): Promise<string>;
    formatSummary(): Promise<string>;
    formatDescription(): Promise<string>;
    formatInlineInput(input: I): Promise<string>;
    formatUsage(): Promise<string>;
    formatInputs(): Promise<string>;
    formatOptionLine(opt: O): Promise<string>;
    /**
     * Insert text before the command's summary.
     *
     * @param meta The metadata of the command.
     */
    formatBeforeSummary(meta: M): Promise<string>;
    /**
     * Insert text after the command's summary.
     *
     * @param meta The metadata of the command.
     */
    formatAfterSummary(meta: M): Promise<string>;
    /**
     * Insert text that appears before an option's summary.
     *
     * @param opt The metadata of the option.
     */
    formatBeforeOptionSummary(opt: O): Promise<string>;
    formatAfterOptionSummary(opt: O): Promise<string>;
    formatOptionDefault(opt: O): Promise<string>;
    formatOptions(): Promise<string>;
    formatOptionsGroup(title: string, options: O[]): Promise<string>;
    formatExamples(): Promise<string>;
    format(): Promise<string>;
}
export interface NamespaceHelpSchema {
    readonly name: string;
    readonly summary: string;
    readonly description: string;
    readonly groups: ReadonlyArray<string>;
    readonly commands: CommandHelpSchema[];
    readonly aliases: ReadonlyArray<string>;
}
export declare class NamespaceSchemaHelpFormatter<C extends ICommand<C, N, M, I, O>, N extends INamespace<C, N, M, I, O>, M extends CommandMetadata<I, O>, I extends CommandMetadataInput, O extends CommandMetadataOption> extends NamespaceHelpFormatter<C, N, M, I, O> {
    format(): Promise<string>;
    serialize(): Promise<NamespaceHelpSchema>;
    formatCommandGroup(commands: ReadonlyArray<HydratedCommandMetadata<C, N, M, I, O>>): Promise<CommandHelpSchema[]>;
    formatCommand(cmd: HydratedCommandMetadata<C, N, M, I, O>): Promise<CommandHelpSchema>;
}
export interface CommandHelpSchemaInput {
    readonly name: string;
    readonly summary: string;
    readonly required: boolean;
}
export interface CommandHelpSchemaOption {
    readonly name: string;
    readonly summary: string;
    readonly groups: ReadonlyArray<string>;
    readonly aliases: ReadonlyArray<string>;
    readonly type: string;
    readonly default?: string | boolean;
    readonly spec: {
        readonly value: string;
    };
}
export interface CommandHelpSchema {
    readonly name: string;
    readonly namespace: ReadonlyArray<string>;
    readonly summary: string;
    readonly description: string;
    readonly groups: ReadonlyArray<string>;
    readonly exampleCommands: ReadonlyArray<string>;
    readonly aliases: ReadonlyArray<string>;
    readonly inputs: ReadonlyArray<CommandHelpSchemaInput>;
    readonly options: ReadonlyArray<CommandHelpSchemaOption>;
}
export declare class CommandSchemaHelpFormatter<C extends ICommand<C, N, M, I, O>, N extends INamespace<C, N, M, I, O>, M extends CommandMetadata<I, O>, I extends CommandMetadataInput, O extends CommandMetadataOption> extends CommandHelpFormatter<C, N, M, I, O> {
    format(): Promise<string>;
    serialize(): Promise<CommandHelpSchema>;
    formatInputs(inputs: ReadonlyArray<I>): Promise<ReadonlyArray<CommandHelpSchemaInput>>;
    formatInput(input: I): Promise<CommandHelpSchemaInput>;
    formatOptions(options: ReadonlyArray<O>): Promise<ReadonlyArray<CommandHelpSchemaOption>>;
    formatOption(option: O): Promise<CommandHelpSchemaOption>;
    formatCommand(cmd: M | HydratedCommandMetadata<C, N, M, I, O>): Promise<CommandHelpSchema>;
}
export declare function createCommandMetadataFromSchema(schema: CommandHelpSchema): Required<CommandMetadata>;
